<?php

/*
 * Copyright (c) 2018 Heimrich & Hannot GmbH
 *
 * @license LGPL-3.0-or-later
 */

namespace HeimrichHannot\ListBundle\Module;

use Contao\Controller;
use Contao\CoreBundle\Framework\ContaoFramework;
use Contao\Module;
use Contao\ModuleModel;
use Contao\System;
use HeimrichHannot\FilterBundle\Config\FilterConfig;
use HeimrichHannot\FilterBundle\Manager\FilterManager;
use HeimrichHannot\FilterBundle\QueryBuilder\FilterQueryBuilder;
use HeimrichHannot\ListBundle\Event\ListCompileEvent;
use HeimrichHannot\ListBundle\Exception\InvalidListConfigException;
use HeimrichHannot\ListBundle\Exception\InvalidListManagerException;
use HeimrichHannot\ListBundle\Lists\ListInterface;
use HeimrichHannot\ListBundle\Manager\ListManagerInterface;
use HeimrichHannot\ListBundle\Model\ListConfigModel;
use HeimrichHannot\ListBundle\Registry\ListConfigRegistry;
use HeimrichHannot\RequestBundle\Component\HttpFoundation\Request;
use Patchwork\Utf8;

class ModuleList extends Module
{
    const TYPE = 'huhlist';

    protected $strTemplate = 'mod_list';

    /**
     * @var ContaoFramework
     */
    protected $framework;

    /**
     * @var ListManagerInterface
     */
    protected $manager;

    /**
     * @var ListConfigModel
     */
    protected $listConfig;

    /**
     * @var FilterConfig
     */
    protected $filterConfig;

    /**
     * @var FilterManager
     */
    protected $filterManager;

    /**
     * @var ListConfigRegistry
     */
    protected $listConfigRegistry;

    /**
     * @var object
     */
    protected $filter;

    /**
     * @var Request
     */
    protected $request;

    /**
     * ModuleList constructor.
     *
     * @param ModuleModel $objModule
     * @param string      $strColumn
     *
     * @throws InvalidListManagerException
     * @throws InvalidListConfigException
     */
    public function __construct(ModuleModel $objModule, $strColumn = 'main')
    {
        $this->framework = System::getContainer()->get('contao.framework');

        parent::__construct($objModule, $strColumn);

        Controller::loadDataContainer('tl_list_config');
        System::loadLanguageFile('tl_list_config');

        $this->listConfigRegistry = System::getContainer()->get('huh.list.list-config-registry');
        $this->filterManager = System::getContainer()->get('huh.filter.manager');
        $this->request = System::getContainer()->get('huh.request');

        // retrieve list config
        $this->listConfig = System::getContainer()->get('huh.list.list-config-registry')->getComputedListConfig((int) $objModule->listConfig);

        $this->manager = System::getContainer()->get('huh.list.util.manager')->getListManagerByName($this->listConfig->manager ?: 'default');
        $this->manager->setListConfig($this->listConfig);
        $this->manager->setModuleData($this->arrData);

        $this->filterConfig = $this->manager->getFilterConfig();
        $this->filter = (object) $this->filterConfig->getFilter();
    }

    public function generate()
    {
        if (System::getContainer()->get('huh.utils.container')->isBackend()) {
            $objTemplate = new \BackendTemplate('be_wildcard');
            $objTemplate->wildcard = '### '.Utf8::strtoupper($GLOBALS['TL_LANG']['FMD'][$this->type][0]).' ###';
            $objTemplate->title = $this->headline;
            $objTemplate->id = $this->id;
            $objTemplate->link = $this->name;
            $objTemplate->href = 'contao/main.php?do=themes&amp;table=tl_module&amp;act=edit&amp;id='.$this->id;

            return $objTemplate->parse();
        }

        if (null === $this->manager) {
            return parent::generate();
        }

        $this->framework->getAdapter(Controller::class)->loadDataContainer('tl_list_config');
        $this->framework->getAdapter(Controller::class)->loadDataContainer($this->filter->dataContainer);
        $this->framework->getAdapter(System::class)->loadLanguageFile($this->filter->dataContainer);

        if (null !== ($listClass = $this->manager->getListByName($this->listConfig->list ?: 'default'))) {
            $reflection = new \ReflectionClass($listClass);

            if (!$reflection->implementsInterface(ListInterface::class)) {
                throw new \Exception(sprintf('List class %s must implement %s', $listClass, ListInterface::class));
            }

            if (!$reflection->implementsInterface(\JsonSerializable::class)) {
                throw new \Exception(sprintf('List class %s must implement %s', $listClass, \JsonSerializable::class));
            }

            $this->manager->setList(new $listClass($this->manager));
        }
        if (true === (bool) $this->manager->getListConfig()->doNotRenderEmpty && empty($this->manager->getList()->getItems())) {
            /** @var FilterQueryBuilder $queryBuilder */
            $queryBuilder = $this->manager->getFilterManager()->getQueryBuilder($this->filter->id);
            $fields = $this->filter->dataContainer.'.* ';

            if ($totalCount = $queryBuilder->select($fields)->execute()->rowCount() < 1) {
                return '';
            }
        }

        return parent::generate();
    }

    /**
     * @return FilterConfig
     */
    public function getFilterConfig(): FilterConfig
    {
        return $this->filterConfig;
    }

    /**
     * @return ListManagerInterface
     */
    public function getManager(): ListManagerInterface
    {
        return $this->manager;
    }

    protected function compile()
    {
        $this->manager->getList()->handleShare();

        // apply module fields to template
        $this->Template->headline = $this->headline;
        $this->Template->hl = $this->hl;

        // add class to every list template
        $cssID = $this->cssID;
        $cssID[1] = $cssID[1].($cssID[1] ? ' ' : '').'huh-list';

        $this->cssID = $cssID;

        $this->Template->noSearch = (bool) $this->manager->getListConfig()->noSearch;

        $this->Template->list = function (string $listTemplate = null, string $itemTemplate = null, array $data = []) {
            return $this->manager->getList()->parse($listTemplate, $itemTemplate, $data);
        };

        System::getContainer()->get('event_dispatcher')->dispatch(ListCompileEvent::NAME, new ListCompileEvent($this->Template, $this, $this->manager->getListConfig()));
    }
}
